<?php

namespace KnitPay\Gateways\Instamojo;

use Pronamic\WordPress\Pay\Core\Gateway as Core_Gateway;
use Exception;
use Pronamic\WordPress\Pay\Payments\Payment;
use WP_Error;

require_once 'lib/Instamojo.php';

/**
 * Title: Instamojo Gateway
 * Copyright: 2020 Knit Pay
 *
 * @author  Knit Pay
 * @version 1.0.0
 * @since   1.0.0
 */
class Gateway extends Core_Gateway {

	/**
	 * Constructs and initializes an Instamojo gateway
	 *
	 * @param Config $config Config.
	 */
	public function __construct( Config $config ) {
		parent::__construct( $config );

		$this->set_method( self::METHOD_HTTP_REDIRECT );

		// Supported features.
		$this->supports = array(
			'payment_status_request',
		);

		$this->test_mode = 0;
		if ( self::MODE_TEST === $config->mode ) {
			$this->test_mode = 1;
		}

	}

	/**
	 * Get supported payment methods
	 *
	 * @see Pronamic_WP_Pay_Gateway::get_supported_payment_methods()
	 */
	public function get_supported_payment_methods() {
		return array(
			PaymentMethods::BANK_TRANSFER,
			PaymentMethods::CREDIT_CARD,
			PaymentMethods::MAESTRO,
			PaymentMethods::INSTAMOJO,
		);
	}

	private function createRequest( Payment $payment, $use_phone ) {

		$api_data    = array();
		$method_data = array();

		/*
		 * New transaction request.
		 */
		$customer = $payment->get_customer();

		$api_data['purpose']    = $payment->get_description();
		$api_data['buyer_name'] = substr(
			trim( ( html_entity_decode( $customer->get_name(), ENT_QUOTES, 'UTF-8' ) ) ),
			0,
			20
		);
		$api_data['email']      = $customer->get_email();
		if ( $use_phone ) {
			if ( empty( $payment->get_billing_address() ) ) {
				return $this->createRequest( $payment, false );
			}
			$api_data['phone'] = $payment->get_billing_address()->get_phone();
		}
		$api_data['amount']       = $payment->get_total_amount()->format();
		$api_data['redirect_url'] = $payment->get_return_url();

		try {
			$this->instamojo_api = new Instamojo( $this->config->client_id, $this->config->client_secret, $this->test_mode );

			$response = $this->instamojo_api->create_payment_request( $api_data );

			if ( isset( $response->id ) ) {
				$method_data['action'] = $response->longurl;
				$method_data['id']     = $response->id;
			}
		} catch ( ValidationException $e ) {
			// handle exceptions releted to response from the server.
			$method_data['errors'] = $e->getErrors();
			foreach ( $method_data['errors'] as $err ) {
				if ( stristr( $err, 'phone' ) ) {
					return $this->createRequest( $payment, false );
				}
				$this->error = new WP_Error( 'instamojo_error', $err );
			}
		} catch ( Exception $e ) {
			// handled common exception messages which will not caught above.
			$this->error = new WP_Error( 'instamojo_error', $e->getMessage() );
		}

		if ( isset( $this->error ) ) {
			return;
		}

		return $method_data;
	}

	/**
	 * Start.
	 *
	 * @see Core_Gateway::start()
	 *
	 * @param Payment $payment Payment.
	 */
	public function start( Payment $payment ) {

		$payment_currency = $payment->get_total_amount()->get_currency()->get_alphabetic_code();
		if ( isset( $payment_currency ) && 'INR' !== $payment_currency ) {
			$currency_error = 'Instamojo only accepts payments in Indian Rupees. If you are a store owner, kindly activate INR currency for ' . $payment->get_source() . ' plugin.';
			$this->error    = new WP_Error( 'instamojo_error', $currency_error );
			return;
		}

		$method_data = $this->createRequest( $payment, true );
		if ( isset( $method_data['action'] ) ) {
			// Update gateway results in payment.
			$payment->set_transaction_id( $method_data['id'] );
			$payment->set_action_url( $method_data['action'] );
		}
	}

	/**
	 * Update status of the specified payment.
	 *
	 * @param Payment $payment Payment.
	 */
	public function update_status( Payment $payment ) {

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		if ( ! isset( $_REQUEST['payment_request_id'] ) || ! isset( $_REQUEST['payment_id'] ) || $_REQUEST['payment_request_id'] !== $payment->get_transaction_id() ) {
			return;
		}

		// phpcs:ignore WordPress.Security.NonceVerification.Recommended
		$payment_id = sanitize_text_field( wp_unslash( $_REQUEST['payment_id'] ) );

		try {
			$this->instamojo_api = new Instamojo( $this->config->client_id, $this->config->client_secret, $this->test_mode );

			$payment_api = $this->instamojo_api->get_payment_by_id( $payment_id );

			$payment_status = $payment_api->status;

			if ( isset( $payment_status ) ) {

				$payment->set_status( Statuses::transform( $payment_status ) );
				$payment->set_transaction_id($payment_id);

			}
		} catch ( Exception $e ) {
			$this->error = new WP_Error( 'instamojo_error', $e->getMessage() );
		}
	}
}
