<?php
namespace KnitPay\Gateways\PayUmoney;

use Pronamic\WordPress\Pay\Core\Gateway as Core_Gateway;

use Exception;
use Pronamic\WordPress\Pay\Payments\Payment;
use WP_Error;


/**
 * Title: PayUMoney Gateway
 * Copyright: 2020 Knit Pay
 *
 * @author Knit Pay
 * @version 1.0.0
 * @since   1.0.0
 */
class Gateway extends Core_Gateway {


	/**
	 * Client.
	 *
	 * @var Client
	 */
	private $client;

	/**
	 * Constructs and initializes an PayUMoney gateway
	 *
	 * @param Config $config
	 *            Config.
	 */
	public function __construct( Config $config ) {
		parent::__construct( $config );

		$this->set_method( self::METHOD_HTTP_REDIRECT );

		// Supported features.
		$this->supports = array(
			'payment_status_request',
		);

		// Client.
		$this->client = new Client( $config );

		if ( self::MODE_TEST === $config->mode ) {
			$this->client->set_payment_server_url( Client::TEST_URL );
		}
	}

	/**
	 * Get supported payment methods
	 *
	 * @see Pronamic_WP_Pay_Gateway::get_supported_payment_methods()
	 */
	public function get_supported_payment_methods() {
		return array(
			PaymentMethods::BANK_TRANSFER,
			PaymentMethods::CREDIT_CARD,
			PaymentMethods::MAESTRO,
			PaymentMethods::PAYUMONEY,
		);
	}

	/**
	 * Start.
	 *
	 * @see Core_Gateway::start()
	 *
	 * @param Payment $payment
	 *            Payment.
	 */
	public function start( Payment $payment ) {
		$payment_currency = $payment->get_total_amount()->get_currency()->get_alphabetic_code();
		if ( isset( $payment_currency ) && 'INR' !== $payment_currency ) {
			$currency_error = 'PayUMoney only accepts payments in Indian Rupees. If you are a store owner, kindly activate INR currency for ' . $payment->get_source() . ' plugin.';
			$this->error    = new WP_Error( 'payumoney_error', $currency_error );
			return;
		}

		$merchant_key  = $this->config->merchant_key;
		$merchant_salt = $this->config->merchant_salt;

		$txnid        = $payment->get_id();
		$amount       = $payment->get_total_amount()->format();
		$product_info = $payment->get_description();

		$customer   = $payment->get_customer();
		$first_name = $customer->get_name()->get_first_name();
		$last_name  = $customer->get_name()->get_last_name();
		$email      = $customer->get_email();
		$phone      = '';
		if ( null !== $customer->get_phone() ) {
			$phone = $customer->get_phone();
		}

		$billing_address = $payment->get_billing_address();
		$address         = $billing_address->get_line_1();
		$address_2       = $billing_address->get_line_2();
		$city            = $billing_address->get_city();
		$state           = $billing_address->get_region();
		$country         = $billing_address->get_country();
		$zipcode         = $billing_address->get_postal_code();

		$udf1 = null;
		$udf2 = null;
		$udf3 = null;
		$udf4 = null;
		$udf5 = null;

		$str = "{$merchant_key}|{$txnid}|{$amount}|{$product_info}|{$first_name}|{$email}|{$udf1}|{$udf2}|{$udf3}|{$udf4}|{$udf5}||||||{$merchant_salt}";

		$hash = strtolower( hash( 'sha512', $str ) );

		$return_url = $payment->get_return_url();
		$cancel_url = $payment->get_return_url();

		$data = array(
			'key'              => $merchant_key,
			'txnid'            => $txnid,
			'amount'           => $amount,
			'productinfo'      => $product_info,
			'firstname'        => $first_name,
			'lastname'         => $last_name,
			'address1'         => $address,
			'address2'         => $address_2,
			'city'             => $city,
			'state'            => $state,
			'country'          => $country,
			'zipcode'          => $zipcode,
			'email'            => $email,
			'phone'            => $phone,
			'surl'             => $return_url,
			'furl'             => $cancel_url,
			'hash'             => $hash,
			'service_provider' => 'payu_paisa',
		);

		// Update gateway results in payment.
		$payment->set_transaction_id( $txnid );
		try {
			$payment->set_action_url( $this->client->get_action_url( $data ) );
		} catch ( \Exception $e ) {
			$this->error = new WP_Error( 'payumoney_error', $e->getMessage() );
		}
	}

	/**
	 * Update status of the specified payment.
	 *
	 * @param Payment $payment
	 *            Payment.
	 */
	public function update_status( Payment $payment ) {

		$status      = $_POST['status'];
		$firstname   = $_POST['firstname'];
		$amount      = $_POST['amount'];
		$txnid       = $_POST['txnid'];
		$posted_hash = $_POST['hash'];
		$key         = $_POST['key'];
		$productinfo = $_POST['productinfo'];
		$email       = $_POST['email'];
		$payuMoneyId = $_POST['payuMoneyId'];

		$merchant_key  = $this->config->merchant_key;
		$merchant_salt = $this->config->merchant_salt;

		if ( ! ( $payuMoneyId === $payment->get_transaction_id() || $txnid === $payment->get_transaction_id() ) || $key !== $merchant_key ) {
			return;
		}

		// Salt should be same Post Request
		if ( isset( $_POST['additionalCharges'] ) ) {
			$additionalCharges = $_POST['additionalCharges'];
			$retHashSeq        = $additionalCharges . '|' . $merchant_salt . '|' . $status . '|||||||||||' . $email . '|' . $firstname . '|' . $productinfo . '|' . $amount . '|' . $txnid . '|' . $key;
		} else {
			$retHashSeq = $merchant_salt . '|' . $status . '|||||||||||' . $email . '|' . $firstname . '|' . $productinfo . '|' . $amount . '|' . $txnid . '|' . $key;
		}
		$hash = hash( 'sha512', $retHashSeq );

		if ( $hash != $posted_hash ) {
			$this->error = new WP_Error( 'payumoney_error', 'Invalid Transaction. Please try again' );
		} else {
			$payment->set_status( Statuses::transform( $status ) );
			$payment->set_transaction_id( $payuMoneyId );
		}
	}
}
