<?php
namespace KnitPay\Gateways\Razorpay;

use Pronamic\WordPress\Pay\Core\Gateway as Core_Gateway;

use Exception;
use Pronamic\WordPress\Pay\Payments\Payment;
use WP_Error;
use Razorpay\Api\Api;


/**
 * Title: Razorpay Gateway
 * Copyright: 2020 Knit Pay
 *
 * @author Knit Pay
 * @version 1.0.0
 * @since   1.7.0
 */
class Gateway extends Core_Gateway {

	/**
	 * Constructs and initializes an Razorpay gateway
	 *
	 * @param Config $config
	 *            Config.
	 */
	public function __construct( Config $config ) {
		parent::__construct( $config );

		$this->set_method( self::METHOD_HTTP_REDIRECT );

		// Supported features.
		$this->supports = array(
		    'payment_status_request', 'recurring_credit_card'
		);
		
		$this->ENV = 'prod';
		if ( self::MODE_TEST === $config->mode ) {
		    $this->ENV = 'test';
		}
	}

	/**
	 * Get supported payment methods
	 *
	 * @see Pronamic_WP_Pay_Gateway::get_supported_payment_methods()
	 */
	public function get_supported_payment_methods() {
		return array(
			PaymentMethods::BANK_TRANSFER,
			PaymentMethods::CREDIT_CARD,
			PaymentMethods::MAESTRO,
		    PaymentMethods::RAZORPAY,
		);
	}

	/**
	 * Start.
	 *
	 * @see Core_Gateway::start()
	 *
	 * @param Payment $payment
	 *            Payment.
	 */
	public function start( Payment $payment ) {
		$payment_currency = $payment->get_total_amount()->get_currency()->get_alphabetic_code();
		// TODO: Check for other currency
		if ( isset( $payment_currency ) && 'INR' !== $payment_currency ) {
			$currency_error = 'Razorpay only accepts payments in Indian Rupees. If you are a store owner, kindly activate INR currency for ' . $payment->get_source() . ' plugin.';
			$this->error    = new WP_Error( 'razorpay_error', $currency_error );
			return;
		}

		$key_id  = $this->config->key_id;
		$key_secret = $this->config->key_secret;
		
		$api = new Api($key_id, $key_secret);
		
		$customer = $payment->get_customer();
		
		// TODO: Add line items
		$link_data = array(
		    'type' => 'link',
		    'amount' => $payment->get_total_amount()->format() * 100,
		    'currency' => $payment_currency,
		    'reminder_enable' => false,
		    'sms_notify' => false,
		    'email_notify' => false,
		    'description' => $payment->get_description(),
		    'customer' => array(
		        'name' => substr(trim( ( html_entity_decode( $customer->get_name(), ENT_QUOTES, 'UTF-8' ) ) ), 0, 45),
		        'email' => $customer->get_email(),
		        'contact' => $payment->get_billing_address()->get_phone(),
		    ),
		    'notes' => array(
		        'knitpay_payment_id' => $payment->id
		    ),
		    'callback_url' => $payment->get_return_url(),
		    'callback_method' => 'get',
		);
		
		$link = $api->invoice->create(
		    $link_data
		    );

		// Update gateway results in payment.
		$payment->set_transaction_id( $link->__get('id') );
		$payment->set_action_url($link->__get('short_url') );
	}

	/**
	 * Update status of the specified payment.
	 *
	 * @param Payment $payment
	 *            Payment.
	 */
	public function update_status( Payment $payment ) {
	    if (!isset($_GET['razorpay_invoice_id'])){
	        wp_safe_redirect( home_url() );
	        exit;
	    }
	    
	    if ($_GET['razorpay_invoice_id'] != $payment->get_transaction_id()) {
	        wp_safe_redirect( home_url() );
	        exit;
	    }
	    
	    $key_id  = $this->config->key_id;
	    $key_secret = $this->config->key_secret;
	    
	    $razorpay_invoice_status = $_GET['razorpay_invoice_status'];
	    $razorpay_payment_id = $_GET['razorpay_payment_id'];
	    $razorpay_invoice_id = $_GET['razorpay_invoice_id'];
	    $razorpay_invoice_receipt =  $_GET['razorpay_invoice_receipt'];
	    
	    $signature_payload = $razorpay_invoice_id . '|' . $razorpay_invoice_receipt . '|' . $razorpay_invoice_status . '|' . $razorpay_payment_id;
	   
	    $expected_signature = hash_hmac('sha256', $signature_payload, $key_secret);
	    
	    if (0 != strcmp($expected_signature, $_GET['razorpay_signature'])) {
	        // TODO: show error
	        wp_safe_redirect( home_url() );
	        exit;
	    }
	    
	    if ( isset( $razorpay_invoice_status ) ) {
	        $payment->set_status( Statuses::transform( $razorpay_invoice_status ) );
	        $payment->set_transaction_id($razorpay_payment_id);
	    }
	}
}
